﻿using Battleship.Exceptions;

namespace Battleship.CollectionGenericObjects;

public class ListGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Список объектов, которые храним
    /// </summary>
    private readonly List<T?> _collection;

    /// <summary>
    /// Максимально допустимое число объектов в списке
    /// </summary>
    private int _maxCount;

    public int Count => _collection.Count;

    public int MaxCount { get { return _collection.Count; } set { if (value > 0) { _maxCount = value; } } }

    public CollectionType GetCollectionType => CollectionType.List;

    /// <summary>
    /// Конструктор
    /// </summary>
    public ListGenericObjects()
    {
        _collection = new();
    }

    public T? Get(int position)
    {
        // проверка позиции
        // выброс ошибки, если выход за границы списка
        if (position < 0 || position > _maxCount) throw new PositionOutOfCollectionException(position);

        return _collection[position];
    }

    public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
    {
        // TODO выброс ошибки, если такой объект есть в коллекции
        if (comparer != null && _collection.Contains(obj, comparer))
        {
            throw new CollectionAlreadyExistsException();
        }
        // проверка, что не превышено максимальное количество элементов
        if (_collection.Count >= _maxCount)
        {
            throw new CollectionOverflowException(_maxCount);
        }
        // вставка в конец набора
        _collection.Add(obj);
        return _maxCount;
    }

    public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
    {
        // TODO выброс ошибки, если такой объект есть в коллекции
        if (comparer != null && _collection.Contains(obj, comparer))
        {
            throw new CollectionAlreadyExistsException();
        }
        // проверка, что не превышено максимальное количество элементов
        if (Count >= _maxCount)
            throw new CollectionOverflowException(_maxCount);

        // проверка позиции
        if (position < 0 || position >= _maxCount)
            throw new PositionOutOfCollectionException(position);

        // вставка по позиции
        _collection.Insert(position, obj);
        return position;
    }

    public T? Remove(int position)
    {
        // проверка позиции
        if (position < 0 || position > _maxCount) throw new PositionOutOfCollectionException(position);
        // удаление объекта из списка
        T temp = _collection[position];
        _collection[position] = null;
        return temp;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Count; i++)
        {
            yield return _collection[i];
        }
    }

    public void CollectionSort(IComparer<T?> comparer)
    {
        _collection.Sort(comparer);
    }
}