﻿using Battleship.CollectionGenericObjects;
using Battleship.Drawings;
using Battleship.Exceptions;

namespace Battleship.CollectionGenericObjects;

/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Массив объектов, которые храним
    /// </summary>
    private T?[] _collection;

    public int Count => _collection.Length;

    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }

        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }

    public CollectionType GetCollectionType => CollectionType.Massive;

    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }

    public T? Get(int position)
    {
        // проверка позиции
        if (position >= _collection.Length || position < 0)
        {
            throw new PositionOutOfCollectionException(position);
        }
        return _collection[position];
    }

    public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
    {
        // TODO выброс ошибки, если такой объект есть в коллекции
        if (comparer != null && _collection.Contains(obj, comparer))
        {
            throw new CollectionAlreadyExistsException();
        }
        // вставка в свободное место набора
        int index = 0;
        while (index < _collection.Length)
        {
            if (_collection[index] == null)
            {
                _collection[index] = obj;
                return index;
            }
            index++;
        }
        throw new CollectionOverflowException(Count);
    }

    public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
    {

        // TODO выброс ошибки, если такой объект есть в коллекции
        if (comparer != null && _collection.Contains(obj, comparer))
        {
            throw new CollectionAlreadyExistsException();
        }

        // проверка позиции
        if (position >= _collection.Length || position < 0)
            throw new PositionOutOfCollectionException(position);

        // проверка, что элемент массива по этой позиции пустой, если нет, то
        if (_collection[position] != null)
        {
            // проверка, что после вставляемого элемента в массиве есть пустой элемент
            int nullIndex = -1;
            for (int i = position + 1; i < Count; i++)
            {
                if (_collection[i] == null)
                {
                    nullIndex = i;
                    break;
                }
            }
            // Если пустого элемента нет, то выходим
            if (nullIndex < 0)
            {
                return -1;
            }
            // сдвиг всех объектов, находящихся справа от позиции до первого пустого элемента
            int j = nullIndex - 1;
            while (j >= position)
            {
                _collection[j + 1] = _collection[j];
                j--;
            }
            throw new CollectionOverflowException(Count);
        }
        // вставка по позиции
        _collection[position] = obj;
        return position;
    }

    public T? Remove(int position)
    {
        // проверка позиции
        // удаление объекта из массива, присвоив элементу массива значение null
        if (position >= _collection.Length || position < 0)
        {
            throw new PositionOutOfCollectionException(position);
        }
        if (_collection[position] == null)
        {
            throw new ObjectNotFoundException(position);
        }
        T temp = _collection[position];
        _collection[position] = null;
        return temp;
    }

    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; i++)
        {
            yield return _collection[i];
        }
    }

    public void CollectionSort(IComparer<T?> comparer)
    {
        List<T?> lst = [.. _collection];
        lst.Sort(comparer.Compare);
        for (int i = 0; i < _collection.Length; ++i)
        {
            _collection[i] = lst[i];
        }
    }
}